'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _extends = _interopDefault(require('@babel/runtime/helpers/extends'));
var React = _interopDefault(require('react'));

function create(fn) {
  var listeners = [];
  var state = {
    current: fn(function (merge) {
      if (typeof merge === 'function') {
        merge = merge(state.current);
      }

      state.current = _extends({}, state.current, merge);
      listeners.forEach(function (listener) {
        return listener(state.current);
      });
    }, function () {
      return state.current;
    })
  };
  return [// useStore
  function (selector, dependencies) {
    var selected = selector ? selector(state.current) : _extends({}, state.current); // Using functional initial b/c selected itself could be a function

    var _React$useState = React.useState(function () {
      return selected;
    }),
        slice = _React$useState[0],
        set = _React$useState[1];

    var sliceRef = React.useRef();
    React.useEffect(function () {
      return void (sliceRef.current = slice);
    }, [slice]);
    React.useEffect(function () {
      var ping = function ping() {
        // Get fresh selected state
        var selected = selector ? selector(state.current) : _extends({}, state.current); // If state is not equal from the get go and not an atomic then shallow equal it

        if (sliceRef.current !== selected && typeof selected === 'object' && !Array.isArray(selected)) {
          selected = Object.entries(selected).reduce(function (acc, _ref) {
            var _extends2;

            var key = _ref[0],
                value = _ref[1];
            return sliceRef.current[key] !== value ? _extends({}, acc, (_extends2 = {}, _extends2[key] = value, _extends2)) : acc;
          }, sliceRef.current);
        } // Using functional initial b/c selected itself could be a function


        if (sliceRef.current !== selected) {
          // Refresh local slice
          set(function () {
            return selected;
          });
        }
      };

      listeners.push(ping);
      return function () {
        return listeners = listeners.filter(function (i) {
          return i !== ping;
        });
      };
    }, dependencies || [selector]); // Returning the selected state slice

    return selected;
  }, {
    subscribe: function subscribe(fn) {
      listeners.push(fn);
      return function () {
        return listeners = listeners.filter(function (i) {
          return i !== fn;
        });
      };
    },
    getState: function getState() {
      return state.current;
    },
    destroy: function destroy() {
      listeners = [];
      state.current = {};
    }
  }];
}

module.exports = create;
